
def on_payload_selection(product, revision, local_elf_path, local_src_path):

    """
    This function is called in Detect Client (inside the Docker container) when a payload has been selected and before the 
    viewer tool has been started. Users can customize this function to download build artifacts (such as elf files) using
    curl or wget - both are available in the Client image.

    The following arguments are provided by the client:
    
    * product: The Product Name, as specified in the Detect Server configuration (based on ProductID in alert data)
    * revision: The Revision field from the alert metadata, typically containing the firmware revision or a build ID.
    * local_elf_path: The docker-internal path where the viewer tools are expecting to find the ELF file.
    * local_src_path: The docker-internal path where the viewer tools are expecting to find the root of the source directory

    The product and revision arguments allow for identifying the right files to download, matching the device that sent the alert.

    The downloaded files should be stored in the paths given by local_elf_path and local_src_path. You can modify them
    by the corresponding settings in the start script, i.e., ELF_REL_PATH and SRC_REL_PATH. 

    By including the ${product} and ${revision} variables in ELF_REL_PATH and SRC_REL_PATH, your downloaded files will
    get unique paths that can be reused for multiple alerts from the same build. This is included in the code example below,
    where the initial if-statement checks if the file is already available and then skips the download.
    
    Example:

    import os
    from urllib.parse import quote

    if (not os.path.isfile(local_elf_path)):
        print("Downloading " + local_elf_path)
        elf_url = "https://[object-storage]/repo/" + quote(product) + "/" + quote(revision) + "/image.elf"
        print("wget -O " + local_elf_path + " " + elf_url)
   else:    
        print("ELF file already exists locally.")
    
   Note the use of the quote function when constructing the url. It encodes the argument strings properly, for example space characters.

   """

pass

