#!/bin/bash

# Update these paths to match your setup, or leave as-is to use the default demo settings.

# The path to the device log file containing alert data (i.e. the input).
# export DEVICE_OUTPUT_LOGFILE=...

# Where to output the alert files, i.e. where the server is looking for data. Can be a network drive.
# export ALERTS_DIR=...

# Note: Only applied if DFM is initialized using xDfmInitializeForLocalUse().
# In that case, this parameter overrides the (dummy) DeviceID set by DFM.
# This is not used in the provided test data in the test-data folder.
# export DEVICE_ID=MyDevice

# Default demo paths (update these paths if necessary)
DEFAULT_ALERTS_DIR="$(pwd)/../test-data/alert-files"
DEFAULT_DEVICE_OUTPUT_LOGFILE="$(pwd)/../test-data/device-log-stm32l4-stopwatch.txt"
DEFAULT_DEVICE_ID="DemoDevice"

# Set ALERTS_DIR to default if not provided
if [ -z "$ALERTS_DIR" ]; then
  ALERTS_DIR=$DEFAULT_ALERTS_DIR
  echo "ALERTS_DIR: Using default directory, $(realpath $ALERTS_DIR)."
fi

# Set DEVICE_OUTPUT_LOGFILE to default if not provided
if [ -z "$DEVICE_OUTPUT_LOGFILE" ]; then
  DEVICE_OUTPUT_LOGFILE=$DEFAULT_DEVICE_OUTPUT_LOGFILE
  echo "DEVICE_OUTPUT_LOGFILE: Using default file, $(realpath $DEVICE_OUTPUT_LOGFILE)."
fi

# Set DEVICE_ID to default if not provided
if [ -z "$DEVICE_ID" ]; then
  DEVICE_ID=$DEFAULT_DEVICE_ID
  echo "DEVICE_ID: Using default ID, $DEVICE_ID."
fi

# Display settings
echo "Percepio Receiver"
echo " Reading from: $DEVICE_OUTPUT_LOGFILE"
echo " Writing to: $ALERTS_DIR"
echo " DeviceID: $DEVICE_ID"
echo " Quit using Ctrl-C"

# Execute the Python script with provided or default arguments
python3 percepio-receiver.py --upload file --folder "$ALERTS_DIR" --eof wait --deviceid "$DEVICE_ID" "$DEVICE_OUTPUT_LOGFILE"
